library autoellipsis;
import 'package:flutter/material.dart';

class Ellipsis {

  Ellipsis(
    this.text,
    this.maxWidth,
   {this.byWord = true,
    this.position = EllipsisFormat.End,}
      );

  final Text text;
  final double maxWidth;
  final EllipsisFormat position;
  final bool byWord;


  static Size textWidth(Text text) {

    final TextPainter textPainter = TextPainter(
        text: TextSpan(text: text.data, style: text.style),
        maxLines: 1,
        textDirection: TextDirection.ltr)
      ..layout(minWidth: 0, maxWidth: double.infinity);
    return textPainter.size;
  }

  static bool _checkText(Text text, double maxWidth){
    if (text.data.isEmpty || text == null) {
      return false;
    }

    if (text.style == null) {
      return false;
    }


    if (maxWidth == null || maxWidth == double.infinity) {
      return false;
    }

    if (textWidth(text).width <= maxWidth) {
      return false;
    }

    return true;
  }

  static Text _newText(Text text, {String data, double fontSize}){
    return new Text(
      data == null ? text.data : data,
      key: text.key,
      style: new TextStyle(
        inherit: text.style.inherit,
        backgroundColor: text.style.backgroundColor,
        fontSize: fontSize == null ? text.style.fontSize : fontSize,
        fontWeight: text.style.fontWeight,
        fontStyle: text.style.fontStyle,
        letterSpacing: text.style.letterSpacing,
        wordSpacing: text.style.wordSpacing,
        textBaseline: text.style.textBaseline,
        height: text.style.height,
        locale: text.style.locale,
        foreground: text.style.foreground,
        background: text.style.background,
        shadows: text.style.shadows,
        fontFeatures: text.style.fontFeatures,
        decoration: text.style.decoration,
        decorationColor: text.style.decorationColor,
        decorationStyle: text.style.decorationStyle,
        decorationThickness: text.style.decorationThickness,
        debugLabel: text.style.debugLabel,
        fontFamily: text.style.fontFamily,
        fontFamilyFallback: text.style.fontFamilyFallback,
      ),
      strutStyle: text.strutStyle,
      textAlign: text.textAlign,
      textDirection: text.textDirection,
      locale: text.locale,
      softWrap: text.softWrap,
      overflow: text.overflow,
      textScaleFactor: text.textScaleFactor,
      maxLines: text.maxLines,
      semanticsLabel: text.semanticsLabel,
      textWidthBasis: text.textWidthBasis,
    );
  }


  static Text cutText(Text text, double maxWidth, {EllipsisFormat position = EllipsisFormat.End, bool byWord = true}) {

    if(!_checkText(text, maxWidth)){
      return text;
    }

    if (position == EllipsisFormat.None) {
      return text;
    }

    List<String> words = byWord ? text.data.split(' ') : null;
    int cut = byWord ? words.length : text.data.length;
    int start = 0;
    int end = cut;
    bool grow = true;
    String fit;

    while (cut > 1) {
      cut -= cut ~/ 2;
      int mult = grow ? 1 : -1;

      switch (position) {
        case EllipsisFormat.End:
          start += mult * cut;
          break;
        case EllipsisFormat.Start:
          end -= mult * cut;
          break;
        default:
          start += mult * (cut ~/ 2);
          end -= mult * (cut - (cut ~/ 2));
          break;
      }

      fit = byWord ? '${words.getRange(0, start).join(' ')}...${words.getRange(
          end, words.length).join(' ')}' :
      '${text.data.substring(0, start)}...${text.data.substring(end, text.data.length)}';

      grow = textWidth(Text(fit, style: text.style,)).width <= maxWidth ? true : false;
    }

    return _newText(text, data: fit);
  }

  static Text compactText(Text text, double maxWidth) {

  double newSize = text.style.fontSize * (maxWidth / textWidth(text).width);
    
    return _newText(text, fontSize: newSize);
  }

}



enum EllipsisFormat
{
  None,
  End,
  Start,
  Middle,
}
